@@ -1,6 +1,6 @@
 /*
  *   stunnel       TLS offloading and load-balancing proxy
- *   Copyright (C) 1998-2023 Michal Trojnara <Michal.Trojnara@stunnel.org>
+ *   Copyright (C) 1998-2024 Michal Trojnara <Michal.Trojnara@stunnel.org>
  *
  *   This program is free software; you can redistribute it and/or modify it
  *   under the terms of the GNU General Public License as published by the
@@ -553,6 +553,12 @@ NOEXPORT void ssl_start(CLI *c) {
     }
     if(c->opt->option.client) {
 #ifndef OPENSSL_NO_TLSEXT
+#ifndef OPENSSL_NO_OCSP
+        if(!SSL_set_tlsext_status_type(c->ssl, TLSEXT_STATUSTYPE_ocsp)) {
+            sslerror("OCSP: SSL_set_tlsext_status_type");
+            throw_exception(c, 1);
+        }
+#endif /* !defined(OPENSSL_NO_OCSP) */
         /* c->opt->sni should always be initialized at this point,
          * either explicitly with "sni"
          * or implicitly with "protocolHost" or "connect" */
@@ -663,7 +669,7 @@ NOEXPORT void ssl_start(CLI *c) {
             }
         }
         if(SSL_session_reused(c->ssl)) {
-            print_session_id(sess);
+            print_session_id(sess); /* otherwise printed from sess_new_cb() */
         } else { /* a new session was negotiated */
             /* SSL_SESS_CACHE_NO_INTERNAL_STORE prevented automatic caching */
             if(!c->opt->option.client)
@@ -750,10 +756,19 @@ NOEXPORT void print_cipher(CLI *c) { /* print negotiated cipher */
     if(c->opt->log_level<LOG_INFO) /* performance optimization */
         return;
 
-    s_log(LOG_INFO, "TLS %s: %s",
-        c->opt->option.client ? "connected" : "accepted",
-        SSL_session_reused(c->ssl) && !c->flag.psk ?
-            "previous session reused" : "new session negotiated");
+#ifndef OPENSSL_NO_PSK
+    if(c->flag.psk_found) {
+        if(c->opt->option.client) {
+            s_log(LOG_ERR, "INTERNAL ERROR: PSK found on a client");
+        } else {
+            s_log(LOG_INFO, "TLS accepted: PSK");
+        }
+    } else
+#endif /* !defined(OPENSSL_NO_PSK) */
+        s_log(LOG_INFO, "TLS %s: %s",
+            c->opt->option.client ? "connected" : "accepted",
+            SSL_session_reused(c->ssl) ?
+                "previous session reused" : "new session negotiated");
 
     cipher=(SSL_CIPHER *)SSL_get_current_cipher(c->ssl);
     s_log(LOG_INFO, "%s ciphersuite: %s (%d-bit encryption)",
@@ -1312,7 +1327,7 @@ NOEXPORT void auth_user(CLI *c) {
         s_log(LOG_WARNING, "Unknown service 'auth': using default 113");
         ident.in.sin_port=htons(113);
     }
-    if(s_connect(c, &ident, addr_len(&ident)))
+    if(s_connect(c, &ident, addr_len(&ident), c->opt->timeout_connect))
         throw_exception(c, 1);
     s_log(LOG_DEBUG, "IDENT server connected");
     remote_port=ntohs(c->peer_addr.in.sin_port);
@@ -1548,7 +1563,8 @@ NOEXPORT SOCKET connect_remote(CLI *c) {
         c->idx=(idx_start+idx_try)%c->connect_addr.num;
         if(!connect_init(c, c->connect_addr.addr[c->idx].sa.sa_family) &&
                 !s_connect(c, &c->connect_addr.addr[c->idx],
-                    addr_len(&c->connect_addr.addr[c->idx]))) {
+                    addr_len(&c->connect_addr.addr[c->idx]),
+                    c->opt->timeout_connect)) {
             if(c->ssl) {
                 SSL_SESSION *sess=SSL_get1_session(c->ssl);
                 if(sess) {
