@@ -1,6 +1,6 @@
 /*
  *   stunnel       TLS offloading and load-balancing proxy
- *   Copyright (C) 1998-2023 Michal Trojnara <Michal.Trojnara@stunnel.org>
+ *   Copyright (C) 1998-2024 Michal Trojnara <Michal.Trojnara@stunnel.org>
  *
  *   This program is free software; you can redistribute it and/or modify it
  *   under the terms of the GNU General Public License as published by the
@@ -140,18 +140,29 @@ typedef long SSL_OPTIONS_TYPE;
 #endif
 
 int context_init(SERVICE_OPTIONS *section) { /* init TLS context */
+    s_log(LOG_DEBUG, "Initializing context [%s]", section->servname);
+
     /* create a new TLS context */
-#if OPENSSL_VERSION_NUMBER>=0x10100000L
 #if OPENSSL_VERSION_NUMBER>=0x30000000L
     section->ctx=SSL_CTX_new_ex(NULL,
         EVP_default_properties_is_fips_enabled(NULL) ?
             "fips=yes" : "provider!=fips",
         section->option.client ?
             TLS_client_method() : TLS_server_method());
-#else /* OPENSSL_VERSION_NUMBER<0x30000000L */
+#elif OPENSSL_VERSION_NUMBER>=0x10100000L
     section->ctx=SSL_CTX_new(section->option.client ?
         TLS_client_method() : TLS_server_method());
-#endif /* OPENSSL_VERSION_NUMBER>=0x30000000L */
+#else /* OPENSSL_VERSION_NUMBER<0x10100000L */
+    section->ctx=SSL_CTX_new(section->option.client ?
+        section->client_method : section->server_method);
+#endif
+    if(!section->ctx) {
+        sslerror("SSL_CTX_new");
+        return 1; /* FAILED */
+    }
+
+    /* set supported protocol versions */
+#if OPENSSL_VERSION_NUMBER>=0x10100000L
     if(section->min_proto_version &&
             !SSL_CTX_set_min_proto_version(section->ctx,
             section->min_proto_version)) {
@@ -166,16 +177,7 @@ int context_init(SERVICE_OPTIONS *section) { /* init TLS context */
             section->max_proto_version);
         return 1; /* FAILED */
     }
-#else /* OPENSSL_VERSION_NUMBER<0x10100000L */
-    if(section->option.client)
-        section->ctx=SSL_CTX_new(section->client_method);
-    else /* server mode */
-        section->ctx=SSL_CTX_new(section->server_method);
-#endif /* OPENSSL_VERSION_NUMBER<0x10100000L */
-    if(!section->ctx) {
-        sslerror("SSL_CTX_new");
-        return 1; /* FAILED */
-    }
+#endif /* OPENSSL_VERSION_NUMBER>=0x10100000L */
 
     /* allow callbacks to access their SERVICE_OPTIONS structure */
     if(!SSL_CTX_set_ex_data(section->ctx, index_ssl_ctx_opt, section)) {
@@ -325,6 +327,12 @@ int context_init(SERVICE_OPTIONS *section) { /* init TLS context */
     if(verify_init(section))
         return 1; /* FAILED */
 
+    /* OCSP stapling */
+#ifndef OPENSSL_NO_OCSP
+    if(ocsp_init(section))
+        return 1; /* FAILED */
+#endif /* OPENSSL_NO_OCSP */
+
     /* initialize the DH/ECDH key agreement */
 #ifndef OPENSSL_NO_TLSEXT
     if(!section->option.client)
@@ -341,6 +349,25 @@ int context_init(SERVICE_OPTIONS *section) { /* init TLS context */
     return 0; /* OK */
 }
 
+/**************************************** cleanup TLS context */
+
+/*
+ * free anything allocate from context_init() and callbacks
+ * also free any cached data allocated in client.c
+ */
+void context_cleanup(SERVICE_OPTIONS *section) {
+    s_log(LOG_DEBUG, "Cleaning up context [%s]", section->servname);
+
+#ifndef OPENSSL_NO_OCSP
+    ocsp_cleanup(section);
+#endif /* !defined(OPENSSL_NO_OCSP) */
+    str_free(section->chain);
+    if(section->session)
+        SSL_SESSION_free(section->session);
+    if(section->ctx)
+        SSL_CTX_free(section->ctx);
+}
+
 /**************************************** SNI callback */
 
 #ifndef OPENSSL_NO_TLSEXT
@@ -713,7 +740,13 @@ NOEXPORT unsigned psk_server_callback(SSL *ssl, const char *identity,
     c=SSL_get_ex_data(ssl, index_ssl_cli);
     found=psk_find(&c->opt->psk_sorted, identity);
     if(!found) {
-        s_log(LOG_INFO, "PSK identity not found (session resumption?)");
+        const char *c=identity;
+        while(*c && isprint(*c))
+            c++;
+        if(*c)
+            s_log(LOG_INFO, "PSK identity not found (session resumption?)");
+        else
+            s_log(LOG_INFO, "PSK identity not found: %s", identity);
         return 0;
     }
     if(found->key_len>max_psk_len) {
@@ -722,7 +755,7 @@ NOEXPORT unsigned psk_server_callback(SSL *ssl, const char *identity,
     }
     memcpy(psk, found->key_val, found->key_len);
     s_log(LOG_NOTICE, "Key configured for PSK identity \"%s\"", identity);
-    c->flag.psk=1;
+    c->flag.psk_found=1;
     return found->key_len;
 }
 
@@ -841,6 +874,21 @@ NOEXPORT int load_pkcs12_file(SERVICE_OPTIONS *section) {
         sslerror("SSL_CTX_use_PrivateKey");
         return 1; /* FAILED */
     }
+#if OPENSSL_VERSION_NUMBER>=0x10002000L
+    if(!SSL_CTX_set0_chain(section->ctx, ca)) {
+        sslerror("SSL_CTX_set0_chain");
+        return 1; /* FAILED */
+    }
+#else /* OPENSSL_VERSION_NUMBER>=0x10002000L */
+    /* FIXME: implement for OpenSSL older than 1.0.2 */
+#if 0
+    /* struct cert_st is private, so the following code won't build */
+    if(section->ctx->cert->key->chain)
+        sk_X509_pop_free(section->ctx->cert->key->chain, X509_free);
+    section->ctx->cert->key->chain=ca;
+#endif
+    sk_X509_pop_free(ca, X509_free); /* just free the memory */
+#endif /* OPENSSL_VERSION_NUMBER>=0x10002000L */
     s_log(LOG_INFO, "Certificate and private key loaded from file: %s",
         section->cert);
     return 0; /* OK */
@@ -1359,7 +1407,7 @@ NOEXPORT void session_cache_save(CLI *c, SSL_SESSION *sess) {
             old=c->opt->connect_session[c->idx];
             c->opt->connect_session[c->idx]=sess;
         } else {
-            s_log(LOG_ERR, "INTERNAL ERROR: Uninitialized client session cache");
+            s_log(LOG_ERR, "INTERNAL ERROR: Uninitialized client session cache (save)");
             old=NULL;
         }
     }
