@@ -3,7 +3,13 @@
 #define WM_VALID_CONFIG     (WM_APP+0)
 #define WM_INVALID_CONFIG   (WM_APP+1)
 #define WM_NEW_CHAIN        (WM_APP+2)
-#define WM_CLIENTS          (WM_APP+3)
+#define WM_TERMINATE        (WM_APP+3)
+
+#define WM_CAPWIN_DESTROY   (WM_APP+0x67)
+#define WM_CAPWIN_NET_DOWN  (WM_APP+0x68)
+#define WM_CAPWIN_NET_UP    (WM_APP+0x69)
+#define WM_CAPWIN_AUTH_OK   (WM_APP+0x70)
+#define WM_CAPWIN_AUTH_FAIL (WM_APP+0x71)
 
 #define IDI_STUNNEL_MAIN    10
 #define IDI_STUNNEL_ACTIVE  11
@@ -13,6 +19,7 @@
 #define IDE_EDIT            20
 #define IDE_PASSEDIT        21
 #define IDE_PINEDIT         22
+#define IDE_PAUSE           23
 
 #define IDM_TRAYMENU        30
 #define IDM_MAINMENU        31
