@@ -1,6 +1,6 @@
 /*
  *   stunnel       TLS offloading and load-balancing proxy
- *   Copyright (C) 1998-2023 Michal Trojnara <Michal.Trojnara@stunnel.org>
+ *   Copyright (C) 1998-2024 Michal Trojnara <Michal.Trojnara@stunnel.org>
  *
  *   This program is free software; you can redistribute it and/or modify it
  *   under the terms of the GNU General Public License as published by the
@@ -266,15 +266,19 @@ NOEXPORT void terminate_threads() {
     threads=0;
     for(c=thread_head; c; c=c->thread_next) /* count client threads */
         threads++;
-    thread_list=str_alloc((threads+1)*sizeof(THREAD_ID));
+    thread_list=str_alloc((threads+2)*sizeof(THREAD_ID));
     i=0;
     for(c=thread_head; c; c=c->thread_next) { /* copy client threads */
         thread_list[i++]=c->thread_id;
         s_log(LOG_DEBUG, "Terminating a thread for [%s]", c->opt->servname);
     }
-    if(cron_thread_id) { /* append cron_thread_id if used */
-        thread_list[threads++]=cron_thread_id;
-        s_log(LOG_DEBUG, "Terminating the cron thread");
+    if(per_second_thread_id) { /* append per_second_thread_id if used */
+        thread_list[threads++]=per_second_thread_id;
+        s_log(LOG_DEBUG, "Terminating the per-second thread");
+    }
+    if(per_day_thread_id) { /* append per_day_thread_id if used */
+        thread_list[threads++]=per_day_thread_id;
+        s_log(LOG_DEBUG, "Terminating the per-day thread");
     }
     CRYPTO_THREAD_unlock(stunnel_locks[LOCK_THREAD_LIST]);
 
@@ -446,7 +450,6 @@ NOEXPORT int accept_connection(SERVICE_OPTIONS *opt, unsigned i) {
 #endif
     if(create_client(fd, s, alloc_client_session(opt, s, s))) {
         s_log(LOG_ERR, "Connection rejected: create_client failed");
-        closesocket(s);
 #ifndef USE_FORK
         service_free(opt);
 #endif
@@ -502,7 +505,7 @@ NOEXPORT void unbind_ports(void) {
             /* FIXME: this is just a crude workaround   */
             /*        is it better to kill the service? */
             /* FIXME: this won't work with FORK threads */
-            opt->option.retry=0;
+            opt->retry=-1; /* disable */
         }
 
         s_log(LOG_DEBUG, "Service [%s] closed", opt->servname);
