@@ -1,6 +1,6 @@
 /*
  *   stunnel       TLS offloading and load-balancing proxy
- *   Copyright (C) 1998-2023 Michal Trojnara <Michal.Trojnara@stunnel.org>
+ *   Copyright (C) 1998-2024 Michal Trojnara <Michal.Trojnara@stunnel.org>
  *
  *   This program is free software; you can redistribute it and/or modify it
  *   under the terms of the GNU General Public License as published by the
@@ -93,8 +93,8 @@ void ui_new_log(const char *line) {
     RETAILMSG(TRUE, (TEXT("%s\r\n"), tstr));
 #else
     /* use UTF-16 or native codepage rather than UTF-8 */
-    _ftprintf(stderr, TEXT("%s\r\n"), tstr);
-    fflush(stderr);
+    _putts(tstr);
+    fflush(stdout);
 #endif
     str_free(tstr);
 }
